import azureml.core
import logging
import os
from helpers import *
import json
import argparse

logging.info("This script was created using version 1.0.10 of the Azure ML SDK")
logging.info("You are currently using version", azureml.core.VERSION, "of the Azure ML SDK")

ap = argparse.ArgumentParser()
ap.add_argument('--subscription_id','-s',help='id of the subscription on which the service will be provider')
ap.add_argument('--resource_group','-g',help='id of the resource group on which the service will be provider')
ap.add_argument('--workspace_name','-w',help='id of the workspace')
ap.add_argument('--region','-r',help='Azure region where the compute happens',default="westeurope")
ap.add_argument('--cluster_name','-c',help='name of the cluster on which to work')
ap.add_argument('--vm_size',help='type of VM on which to run the compute',default="STANDARD_D2_V2")
ap.add_argument('--max_nodes','-n',help='maximum number of nodes allowed on the cluster',default=1)

args = vars(ap.parse_args())

def config(subscription_id=None,resource_group=None,workspace_name=None,region=None,cluster_name=None,vm_size=None,max_nodes=None):

    try : 
        from azureml.core import Workspace
        ws=Workspace.from_config()
        logging.info("found existing workspace on which to work : ",workspace.name)
    except :
        logging.info("creating workspace .... ")
        ws=create_workspace(subscription_id,resource_group,workspace_name,region)
        logging.info("workspace created ! ")
    logging.info("creating cluster...")
    cluster=create_cluster(ws,cluster_name,vm_size,max_nodes)
    return ws,cluster

#create_cluter()
if __name__ == '__main__':
    if os.path.isfile("cluster_config.json") :
        if input("found a local cluster_config.json, do you want to continue with it? (y/n)")=="y":            
            import json 
            with open("cluster_config.json") as jsondata :
                localargs=json.load(jsondata)

            for key in args.keys():
                if (args[key] is None) and (key in localargs.keys()) :
                    args[key]=localargs[key]
    with open("cluster_config.json","w") as file :
        json.dump(args,file)
    config(args["subscription_id"],args["resource_group"],args["workspace_name"],args["region"],args["cluster_name"],args["vm_size"],args["max_nodes"])


def create_workspace(subscription_id,resource_group,workspace_name,region):
    if subscription_id is None :
        subscription_id=input("enter your subscription id : ")
    else : 
        print("working on subscription :" ,subscription_id)
    if resource_group is None :
        resource_group=input("enter your resource groupe id : ")
    else : 
        print("working on resource group :" ,resource_group)
    if workspace_name is None :
        workspace_name=input("enter your workspace name : ")
    else : 
        print("working on workspace :" ,workspace_name)
    from azureml.core import Workspace
    try:
        ws = Workspace(subscription_id = subscription_id, resource_group = resource_group, workspace_name = workspace_name)
        # write the details of the workspace to a configuration file to the notebook library
        ws.write_config()
        print("Workspace configuration succeeded. Skip the workspace creation steps below")
    except:
        ws = Workspace.create(name = workspace_name,
                        subscription_id = subscription_id,
                        resource_group = resource_group, 
                        location = region,
                        create_resource_group = True,
                        exist_ok = True)
        ws.get_details()
        # write the details of the workspace to a configuration file to the notebook library
        ws.write_config()
    return ws

def create_cluster(ws,cluster_name,vm_size,max_nodes):
    if cluster_name is None :
        cluster_name=input("name of the cluster : ")
    else :
        print("using cluster : ",cluster_name)
    if vm_size is None :
        vm_size=input("size of your VM : ")
    else :
        print("using vm size of : ",vm_size)
    if max_nodes is None :
        max_nodes=input("maximum amount of nodes on the cluster : ")
    else :
        print("maximum amount of nodes is : ",max_nodes)

    from azureml.core.compute import ComputeTarget, AmlCompute
    from azureml.core.compute_target import ComputeTargetException

    # Verify that cluster does not exist already
    try:
        cluster = ComputeTarget(workspace=ws, name=cluster_name)
        print("Found existing cluster")
    except ComputeTargetException:
        print("Creating new cluster")        
        # Specify the configuration for the new cluster
        compute_config = AmlCompute.provisioning_configuration(vm_size=vm_size,
                                                            min_nodes=0,
                                                            max_nodes=max_nodes)

        # Create the cluster with the specified name and configuration
        cluster = ComputeTarget.create(ws, cluster_name, compute_config)
        
        # Wait for the cluster to complete, show the output log
        cluster.wait_for_completion(show_output=True)
    return cluster


import torch
import torchvision
import torchvision.transforms as transforms
from torch.optim import lr_scheduler
import torch.optim as optim
import time
import copy
import torch.onnx
device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
print("running on device",device)


### METADATA ###
import numpy as np
with open("cleaning.txt","rb") as file:
    data=file.read()
lines=data.decode("utf8").replace("\r","").split("\n")
metalabels_100=[line.split("\t")[0] for line in lines]  
labels_100=[k for line in lines for k in line.split("\t")[1].split(",") ]

image_size=32
### DATA LOADING ### 
transform = transforms.Compose( 
    [
    transforms.RandomResizedCrop(image_size),
  #  transforms.RandomHorizontalFlip(),
    transforms.ToTensor(),
    transforms.Normalize((0.5, 0.5, 0.5), (0.5, 0.5, 0.5))
    
    ])
trainset = torchvision.datasets.CIFAR100(root='./data', train=True,
                                        download=True, transform=transform)
trainloader = torch.utils.data.DataLoader(trainset, batch_size=4,
    shuffle=True, num_workers=2)
testset = torchvision.datasets.CIFAR100(root='./data', train=False,
                                       download=True, transform=transform)
testloader = torch.utils.data.DataLoader(testset, batch_size=4,
                                         shuffle=False, num_workers=2)

dataloaders={"train" : trainloader, "val" : testloader}

# functions to show an image
import numpy as np


dataiter = iter(trainloader)
images, labels = dataiter.next()
classes = ('plane', 'car', 'bird', 'cat',
           'deer', 'dog', 'frog', 'horse', 'ship', 'truck')
classes_100 = tuple(labels_100)

import torch.nn as nn
import torch.nn.functional as F


class Net(nn.Module):
    def __init__(self):
        super(Net, self).__init__()
        self.conv1 = nn.Conv2d(3, 6, 5)
        self.pool = nn.MaxPool2d(2, 2)
        self.conv2 = nn.Conv2d(6, 16, 5)
        self.fc1 = nn.Linear(16 * 5 * 5, 120)
        self.fc2 = nn.Linear(120, 84)
        self.fc3 = nn.Linear(84, 100)

    def forward(self, x):
        x = self.pool(F.relu(self.conv1(x)))
        x = self.pool(F.relu(self.conv2(x)))
        x = x.view(-1, 16 * 5 * 5)
        x = F.relu(self.fc1(x))
        x = F.relu(self.fc2(x))
        x = self.fc3(x)
        return x


if __name__=="__main__" :
    net = Net()
    net.to(device)
    import torch.optim as optim

    criterion = nn.CrossEntropyLoss()
    optimizer = optim.SGD(net.parameters(), lr=0.001, momentum=0.9)

    for epoch in range(4):  # loop over the dataset multiple times

        running_loss = 0.0
        for i, data in enumerate(trainloader, 0):
            # get the inputs
            inputs, labels = data
            inputs, labels = inputs.to(device), labels.to(device)
            # zero the parameter gradients
            optimizer.zero_grad()

            # forward + backward + optimize
            outputs = net(inputs)
            loss = criterion(outputs, labels)
            loss.backward()
            optimizer.step()

            # print statistics
            running_loss += loss.item()
            if i % 2000 == 1999:    # print every 2000 mini-batches
                print('[%d, %5d] loss: %.3f' %
                    (epoch + 1, i + 1, running_loss / 2000))
                running_loss = 0.0
            
    print('Finished Training')
    correct = 0
    total = 0
    with torch.no_grad():
        for data in testloader:
            images, labels = data
            outputs = net(images)
            _, predicted = torch.max(outputs.data, 1)
            total += labels.size(0)
            correct += (predicted == labels).sum().item()

    print('Accuracy of the network on the 10000 test images: %d %%' % (
        100 * correct / total))

    print("exporting the model")
    dummy_input = torch.randn(1, 3, image_size, image_size)
    
    torch.onnx.export(net,dummy_input,"CIFAR_model.onnx",input_names=["data"],output_names=["classLabel"])
    print(dummy_input,len(dummy_input[0][0]))



import azureml.core
from azureml.core import Experiment
from azureml.core import Workspace
from azureml.core.compute import ComputeTarget, AmlCompute
import time
import logging
from azureml.core.runconfig import DataReferenceConfiguration
time0=time.time()
import argparse
import os
from azureml.core import run
ap = argparse.ArgumentParser()
ap.add_argument('--experiment_name','-e',help='name of your AML experiment')
ap.add_argument('--script','-s',help='name of the file to train')
ap.add_argument("--project_folder", "-p",help='folder where the files will be',default = './train-on-amlcompute')
ap.add_argument('--framework','-f',help="TensorFlow, PyTorch, Chainer, and Python are supported ")
ap.add_argument('--cluster_name','-c',help="Name of the cluster on which to do the training")
ap.add_argument("--storage_account","-d",help='account name for location of data (must be blob here)')
ap.add_argument("--storage_key",help='storage key')
ap.add_argument("--storage_path",help='container in the storage account where the data is')
args = vars(ap.parse_args())

print(args)


def autodetect_framework(script):
    with open(script,"rb") as file :
        data=file.read().decode("utf8").lower()
    if "import torch" in data:
        return "PyTorch"
    elif "import tensorflow" in data:
        return "TensorFlow"
    elif "import chainer" in data:
        return "Chainer"
    else : 
        return "Estimator"

def do_stuff(experiment_name,project_folder,script,framework,cluster_name,storage_account,storage_key,storage_path):
    time0=time.time()
    logging.info("starting to do stuf...")
    from azureml.core.compute import ComputeTarget, AmlCompute

    ### LOADING THE WORKSPACE ###
    try :
        ws = Workspace.from_config()
        logging.info("workspace already created, loading the workspace")
        if cluster_name is None :
            logging.info("hey let's select a random cluster ! (no cluster specified)")
            cluster_name = [k for k in ws.compute_targets.keys()][0]
            logging.info("it worked ! we selected : ",cluster_name)
        cluster = ComputeTarget(workspace=ws, name=cluster_name)
    except : 
        
        go_config=input("workspace not created yet... Do you want to launch the configuration config.py that will create all the resources? (Y/N)")
        while go_config.lower() not in ["y","n"]:
            go_config=input("invalid input, do you want to launch the config (Y/N)")
        if go_config.lower()=="y":
            from config import config
            config()
        else :
            logging.info("ok, exiting")
            return 1
        
    
  
    ### LOADING THE EXPERIMENT ###
    if experiment_name is None :
        experiment_name=input(" experiment name (>3 characters) : ")
        args["experiment_name"]=experiment_name
    else : 
        logging.info("working on experiment : ", experiment_name )

    if experiment_name not in ws.experiments.keys() :
        stay=input("This experiment does not exist, do you want to create the experiment "+str(experiment_name)+"? (Y/N)")
        while stay not in ["Y","N"]:
            stay=input("value must be \"Y\" or \"N\", please enter valid value : ")
            break
        if stay=="N" :
            logging.info("exiting..")
            return ""
        elif stay=="Y":
            logging.info("creating experiment... OK that part doesn't work yet...")
            from azureml.core import Experiment
            experiment = Experiment(workspace=ws, name=experiment_name)
            logging.info("experiment created ! ",experiment_name)
    else :
        from azureml.core import Experiment
        experiment = Experiment(workspace = ws, name = experiment_name)
    
   

    ### LOADING THE TRAIN FILE ###
    import os
    import shutil
    if script is None :
        script=input(" training file : ")
        args["script"]=script
    else:
        logging.info("launching the training of : ",script)

    
    ### LINKING THE DATA ###
    from azureml.core import Datastore

    if storage_account is None :
        storage_account=input("please enter the name of the storage account where you have the data, or pass if your script has no data : ")
        args["storage_account"]=storage_account
    
    if storage_account !="" :
        if storage_path is None :
            storage_path=input("please enter relative path to the container of your data : ")
            args["storage_path"]=storage_path
    
        if storage_key is None :
            storage_key=input("please enter the Access key to access your data : ")
            args["storage_key"]=storage_key
        
        ds = Datastore.register_azure_blob_container(workspace=ws, 
                                                datastore_name='cifar100', 
                                                container_name=storage_path,
                                                account_name=storage_account, 
                                                account_key=storage_key,
                                                create_if_not_exists=True,
                                                overwrite=True)

        script_params = {
        '--data_folder': ds.as_mount()
        }
    else : 
        script_params={}
    logging.info("working in folder ",project_folder)
    os.makedirs(project_folder, exist_ok=True)
    shutil.copy("cleaning.txt",project_folder)
    shutil.copy(script, project_folder)

    ### DOING THE TRAINING
    
    if framework is None:
        framework = autodetect_framework(script).lower()
        logging.info("autodetected framework : ",framework)
    else :
        framework=framework.lower()
        logging.info("didn't find a framework ; ",framework)
    
    if framework=="pytorch": 
        logging.info("detected PyTorch as backend Framework, will work accordingly")  
        from azureml.train.dnn import PyTorch
        estimator=PyTorch(source_directory=project_folder,
                            script_params=script_params,
                            compute_target=cluster,
                            entry_script=script
                        )
    elif framework=="tensorflow" :
        logging.info("detected Tensorflow as backend Framework, will work accordingly")  
        from azureml.train.dnn import TensorFlow
        estimator=TensorFlow(
                        source_directory=project_folder,
                        script_params=script_params,
                        compute_target=cluster,
                        entry_script=script,
                        pip_packages=["h5py"]
                        )
    elif framework == "chainer":
        logging.info("detected Chainer as backend Framework, will work accordingly. The implementation is buggy, though.")  
        from azureml.train.dnn import Chainer
        estimator=Chainer(
                        source_directory=project_folder,
                        script_params=script_params,
                        compute_target=cluster,
                        entry_script=script
                        )           
    else : 
        logging.warning("WARNING : didn't detect PyTorch, Tensorflow, or Chainer. If you are working with those, please input it as --framework (pytorch/tensorflow/chainer)")  
        from azureml.train.estimator import Estimator
        estimator = Estimator(
                        source_directory=project_folder,
                        script_params=script_params,
                        compute_target=cluster,
                        entry_script=script
        )
    run = experiment.submit(estimator)
    run
    # Shows output of the run on stdout.
    run.wait_for_completion(show_output=True)
    run.get_metrics()
    return args

if __name__=="__main__" :
    import json 
    if os.path.isfile("config.json") :
        if input("found a local config.json, do you want to continue with it? (y/n)")=="y":            
            with open("config.json") as jsondata :
                localargs=json.load(jsondata)

            for key in args.keys():
                if (args[key] is None) and (key in localargs.keys()) :
                    args[key]=localargs[key]
    args = do_stuff(args["experiment_name"],args["project_folder"],args["script"],args["framework"],args["cluster_name"],args["storage_account"],args["storage_key"],args["storage_path"])

    with open("config.json","w") as file :
        json.dump(args,file)



import os
from distutils.core import setup
from setuptools import find_packages

here = os.path.dirname(__file__)
packages = find_packages(where=here)
package_dir = {k: os.path.join(here, k.replace(".", "/")) for k in packages}

setup(name='amlfbp',
      version='0.34',
      description="Azure Machine Learning service compute for Busy People. See more in the Github",
      author='Aleksander Callebat',
      author_email='aleks_callebat@hotmail.fr',
      url='https://github.com/alekscallebat/amlfbp',
      packages=["amlfbp"])